"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var fluent_ffmpeg_1 = __importDefault(require("fluent-ffmpeg"));
var fs_extra_1 = require("fs-extra");
var os_1 = __importDefault(require("os"));
var path_1 = require("path");
var shelljs_1 = require("shelljs");
var config_1 = require("../config");
var logger_1 = require("../logger");
var utils_1 = require("../utils");
var Recording = (function () {
    function Recording(savePath) {
        this._fps = 20;
        this._stopped = false;
        this._savePath = savePath;
        this._videoPath = this._savePath + "/video.mp4";
        var platform = os_1.default.platform();
        if (platform !== "linux" && platform !== "darwin") {
            throw new Error("No support for " + platform + ", yet");
        }
        var format = platform === "linux" ? "x11grab" : "avfoundation";
        this._command = fluent_ffmpeg_1.default()
            .addInput(config_1.CONFIG.display + ".0")
            .inputFormat(format)
            .inputOptions([
            "-draw_mouse 0",
            "-r " + this._fps,
            "-copyts"
        ])
            .output(this._videoPath)
            .videoCodec("libx264")
            .outputOptions([
            "-pix_fmt yuv420p",
            "-crf 19",
            "-preset slow",
            "-threads 0",
            "-vsync 0"
        ])
            .format("mp4")
            .on("start", function (command) { return logger_1.logger.debug(command); })
            .on("error", function (err) { return logger_1.logger.debug("ffmpeg: " + err.message); });
        this._command.run();
    }
    Recording.start = function (savePath) {
        return __awaiter(this, void 0, void 0, function () {
            var path, recording;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        path = path_1.resolve(savePath);
                        return [4, fs_extra_1.ensureDir(path)];
                    case 1:
                        _a.sent();
                        recording = new Recording(path);
                        return [2, recording];
                }
            });
        });
    };
    Recording.prototype.stop = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this._stopped)
                            throw new Error("Recorder already stopped.");
                        this._stopped = true;
                        logger_1.logger.debug("Recording: stopping");
                        this._command.ffmpegProc.stdin.write("q");
                        return [4, new Promise(function (resolve) { return setTimeout(resolve, 100); })];
                    case 1:
                        _a.sent();
                        this._command.kill("SIGTERM");
                        return [4, utils_1.sleep(2000)];
                    case 2:
                        _a.sent();
                        logger_1.logger.debug("Recording: stopped");
                        return [4, this.saveGif()];
                    case 3:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    Recording.prototype.saveGif = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2, new Promise(function (resolve) {
                        shelljs_1.cd(_this._savePath);
                        shelljs_1.exec('ffmpeg -i video.mp4 -vf "fps=10,scale=640:-1:flags=lanczos,setpts=0.25*PTS" -c:v pam -f image2pipe - | convert -delay 10 - -loop 0 -layers optimize video.gif', function () {
                            resolve();
                        });
                    })];
            });
        });
    };
    return Recording;
}());
exports.Recording = Recording;
//# sourceMappingURL=Recording.js.map