"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var fixtures_1 = require("@qawolf/fixtures");
var buildJob_1 = require("./buildJob");
var buildTest_1 = require("./buildTest");
describe("buildTest", function () {
    test("builds a test from a job", function () { return __awaiter(void 0, void 0, void 0, function () {
        var events, job, testString;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fixtures_1.loadEvents("login")];
                case 1:
                    events = _a.sent();
                    job = buildJob_1.buildJob(events, "test job");
                    testString = buildTest_1.buildTest(job, false);
                    expect(testString).toMatchSnapshot();
                    return [2];
            }
        });
    }); });
});
describe("formatStep", function () {
    test("formats click on link step", function () {
        var step = {
            action: "click",
            target: {
                tagName: "a",
                textContent: "contact"
            }
        };
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual("click contact link");
    });
    test("formats click on submit input step", function () {
        var step = {
            action: "click",
            target: {
                inputType: "submit",
                tagName: "input",
                textContent: "sign in"
            }
        };
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual('click sign in input[type="submit"]');
    });
    test("formats scroll down action", function () {
        var step = {
            action: "scroll",
            target: { xpath: "scroll" },
            scrollDirection: "down"
        };
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual("scroll down");
    });
    test("formats scroll up action", function () {
        var step = {
            action: "scroll",
            target: { xpath: "scroll" },
            scrollDirection: "up"
        };
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual("scroll up");
    });
    test("formats type into text input", function () {
        var step = {
            action: "input",
            target: {
                id: "input1",
                inputType: "text",
                labels: ["username"],
                name: "user",
                placeholder: "Jane Doe",
                tagName: "input"
            },
            value: "spirit"
        };
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual('enter "spirit" into username input[type="text"]');
    });
    test("formats type into password input", function () {
        var step = {
            action: "input",
            target: {
                id: "input2",
                inputType: "password",
                placeholder: "secret",
                tagName: "input"
            },
            value: "supersecret"
        };
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual('enter "supersecret" into secret input[type="password"]');
    });
    test("shortens target name if needed", function () {
        var step = {
            action: "click",
            target: {
                inputType: "submit",
                tagName: "input",
                textContent: "sign in" + "x".repeat(200)
            }
        };
        var formattedTextContent = "sign in" + "x".repeat(43);
        var formattedStep = buildTest_1.formatStep(step);
        expect(formattedStep).toEqual("click " + formattedTextContent + " input[type=\"submit\"]");
    });
});
//# sourceMappingURL=buildTest.test.js.map