"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var xpath_1 = require("./xpath");
var cleanText = function (text) {
    return text.trim().toLowerCase();
};
exports.getDataValue = function (element, dataAttribute) {
    if (!dataAttribute)
        return null;
    return element.getAttribute(dataAttribute) || null;
};
exports.getLabels = function (element) {
    var labelElements = element.labels;
    if (!labelElements || !labelElements.length)
        return null;
    var labels = [];
    for (var i = 0; i < labelElements.length; i++) {
        var textContent = labelElements[i].textContent;
        if (textContent) {
            labels.push(cleanText(textContent));
        }
    }
    return labels;
};
exports.getParentText = function (element) {
    if (!element.parentElement)
        return null;
    if (element.parentElement.textContent) {
        var parentText = [];
        for (var i = 0; i < element.parentElement.children.length; i++) {
            var textContent = element.parentElement.children[i].textContent;
            if (textContent) {
                parentText.push(cleanText(textContent));
            }
            parentText.push(cleanText(element.parentElement.textContent));
        }
        return parentText;
    }
    return exports.getParentText(element.parentElement);
};
exports.getPlaceholder = function (element) {
    if (element.tagName && element.tagName.toLowerCase() === "select") {
        var options = element.options;
        var placeholderOption = null;
        for (var i = 0; i < options.length; i++) {
            if (options[i].disabled) {
                placeholderOption = options[i];
                break;
            }
        }
        if (!placeholderOption || !placeholderOption.textContent)
            return null;
        return cleanText(placeholderOption.textContent);
    }
    else {
        if (!element.placeholder)
            return null;
        return cleanText(element.placeholder);
    }
};
exports.getTextContent = function (element) {
    if (!element.textContent)
        return null;
    return cleanText(element.textContent);
};
exports.getDescriptor = function (element, dataAttribute) {
    return {
        classList: (element.className || "").length
            ? element.className.split(" ")
            : null,
        dataValue: exports.getDataValue(element, dataAttribute),
        href: element.href || null,
        id: element.id || null,
        inputType: element.type || null,
        labels: exports.getLabels(element),
        name: element.name || null,
        parentText: exports.getParentText(element),
        placeholder: exports.getPlaceholder(element),
        tagName: element.tagName ? element.tagName.toLowerCase() : null,
        textContent: exports.getTextContent(element),
        xpath: xpath_1.getXpath(element)
    };
};
//# sourceMappingURL=element.js.map