"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var element_1 = require("./element");
var xpath_1 = require("./xpath");
var strongMatchKeys = [
    "id",
    "name",
    "labels",
    "placeholder",
    "textContent",
    "xpath"
];
var isNil = function (value) {
    return typeof value === "undefined" || value === null;
};
exports.compareArrays = function (base, compare) {
    if (!base || !base.length || !compare || !compare.length)
        return 0;
    var baseItemsInCompare = base.filter(function (v) { return compare.includes(v); }).length;
    return Math.round((baseItemsInCompare / base.length) * 100);
};
exports.compareDescriptorKey = function (key, targetValue, compareValue) {
    var percent;
    if (isNil(targetValue) || isNil(compareValue)) {
        percent = 0;
    }
    else if (key === "xpath") {
        var resolveSameElement = xpath_1.isXpathEqual(targetValue, compareValue);
        percent = resolveSameElement ? 100 : 0;
    }
    else if (Array.isArray(targetValue) && Array.isArray(compareValue)) {
        percent = exports.compareArrays(targetValue, compareValue);
    }
    else {
        percent = targetValue === compareValue ? 100 : 0;
    }
    return { key: key, percent: percent };
};
exports.compareDescriptors = function (target, compare) {
    var matches = [];
    for (var _i = 0, _a = Object.keys(target); _i < _a.length; _i++) {
        var key = _a[_i];
        var match = exports.compareDescriptorKey(key, target[key], compare[key]);
        if (match.percent > 0)
            matches.push(match);
    }
    return matches;
};
exports.matchElements = function (_a) {
    var dataAttribute = _a.dataAttribute, target = _a.target, elements = _a.elements, _b = _a.requireStrongMatch, requireStrongMatch = _b === void 0 ? false : _b;
    var matches = [];
    elements.forEach(function (element) {
        var descriptor = element_1.getDescriptor(element, dataAttribute);
        var targetMatches = exports.compareDescriptors(target, descriptor);
        var strongMatch = targetMatches.find(function (m) {
            return strongMatchKeys.includes(m.key);
        });
        var majorityMatch = targetMatches.length / Object.keys(target).length > 0.5;
        if (!strongMatch && !majorityMatch) {
            targetMatches = [];
        }
        else if (requireStrongMatch && !strongMatch) {
            targetMatches = [];
        }
        var value = targetMatches.reduce(function (accumulator, match) { return accumulator + match.percent; }, 0);
        if (value) {
            matches.push({ element: element, targetMatches: targetMatches, value: value });
        }
    });
    return matches;
};
exports.topMatch = function (args) {
    var matches = exports.matchElements(args).sort(function (a, b) {
        return a.value > b.value ? -1 : 1;
    });
    if (matches.length <= 0 || matches[0].value <= 0) {
        console.log("no match found");
        return null;
    }
    var equalTopMatches = matches.filter(function (m) { return m.value === matches[0].value; });
    if (equalTopMatches.length > 1) {
        console.log("no match since all top matches are equal", equalTopMatches);
        return null;
    }
    console.log("top match found", matches[0]);
    return matches[0];
};
//# sourceMappingURL=match.js.map