"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var element_1 = require("./element");
var score_1 = require("./score");
var DEFAULT_THRESHOLD = 0.6;
exports.computeSimilarityScores = function (step, elements, dataAttribute) {
    var base = step.element;
    var scores = [];
    for (var i = 0; i < elements.length; i++) {
        var compare = element_1.getDescriptor(elements[i], dataAttribute);
        if (!compare) {
            scores.push(0);
        }
        else {
            var score = score_1.computeSimilarityScore(compare, base);
            scores.push(score);
        }
    }
    return scores;
};
exports.findCandidateElements = function (step) {
    var qualifiedName = step.action === "input" ? "input,textarea,select" : "*";
    var elements = document.querySelectorAll(qualifiedName);
    var candidates = [];
    for (var i = 0; i < elements.length; i++) {
        var element = elements[i];
        var rect = element.getBoundingClientRect();
        if (rect.height && rect.width)
            candidates.push(element);
    }
    return candidates;
};
exports.findElementByDataValue = function (dataAttribute, dataValue) {
    var selector = "[" + dataAttribute + "='" + dataValue + "']";
    var elements = document.querySelectorAll(selector);
    if (elements.length > 1) {
        throw new Error("Can't decide: found " + elements.length + " elements with data attribute " + selector);
    }
    return elements[0] || null;
};
exports.findTopElement = function (step, dataAttribute, threshold) {
    if (threshold === void 0) { threshold = DEFAULT_THRESHOLD; }
    var elements = exports.findCandidateElements(step);
    var scores = exports.computeSimilarityScores(step, elements, dataAttribute);
    var maxScore = Math.max.apply(Math, scores);
    var maxPossibleScore = score_1.computeMaxPossibleScore(step.element);
    if (maxScore / maxPossibleScore < threshold) {
        return null;
    }
    var maxElements = [];
    scores.forEach(function (score, index) {
        if (score < maxScore)
            return;
        maxElements.push(elements[index]);
    });
    if (maxElements.length > 1) {
        return null;
    }
    return maxElements[0];
};
exports.waitForElement = function (step, dataAttribute, timeout, threshold) {
    if (timeout === void 0) { timeout = 30000; }
    if (threshold === void 0) { threshold = DEFAULT_THRESHOLD; }
    return __awaiter(void 0, void 0, void 0, function () {
        return __generator(this, function (_a) {
            return [2, new Promise(function (resolve, reject) {
                    var intervalId = setInterval(function () {
                        var element = null;
                        if (dataAttribute && step.element.dataValue) {
                            element = exports.findElementByDataValue(dataAttribute, step.element.dataValue);
                        }
                        else {
                            element = exports.findTopElement(step, dataAttribute, threshold);
                        }
                        if (element) {
                            console.log("waitForElement: found element", element);
                            clearInterval(intervalId);
                            clearTimeout(rejectId);
                            resolve(element);
                        }
                    }, 100);
                    var rejectId = setTimeout(function () {
                        reject("No element found for step " + JSON.stringify(step));
                        clearInterval(intervalId);
                    }, timeout);
                })];
        });
    });
};
//# sourceMappingURL=rank.js.map